#!/usr/bin/python -u

"""
Copyright (C) 2017 Jacksgong(jacksgong.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from os.path import exists

from ducat.confloader import ConfLoader
from ducat.helper import LOG_LEVELS_MAP, get_conf_path, handle_home_case, print_unicode
from ducat.logprocessor import LogProcessor, indent_wrap
from ducat.logregex import LogRegex
from ducat.terminalcolor import termcolor, RED, RESET, YELLOW, GREEN, colorize, WHITE, allocate_color

__author__ = 'JacksGong'

# Script to highlight adb logcat output for console
# Originally written by Jeff Sharkey, http://jsharkey.org/
# Piping detection and popen() added by other Android team members
# Package filtering and output improvements by Jake Wharton, http://jakewharton.com
# Package adapt for okcat by Jacks Gong, https://jacksgong.com

import sys
import re
import subprocess
import time
from subprocess import PIPE, STDOUT

# noinspection Annotator
PID_LINE = re.compile(r'^\w+\s+(\w+)\s+\w+\s+\w+\s+\w+\s+\w+\s+\w+\s+\w\s([\w|\.|\/]+)')
PID_START = re.compile(r'^.*: Start proc ([a-zA-Z0-9._:]+) for ([a-z]+ [^:]+): pid=(\d+) uid=(\d+) gids=(.*)$')
PID_START_5_1 = re.compile(r'^.*: Start proc (\d+):([a-zA-Z0-9._:]+)/[a-z0-9]+ for (.*)$')
PID_START_DALVIK = re.compile(
    r'^E/dalvikvm\(\s*(\d+)\): >>>>> ([a-zA-Z0-9._:]+) \[ userId:0 \| appId:(\d+) \]$')
PID_KILL = re.compile(r'^Killing (\d+):([a-zA-Z0-9._:]+)/[^:]+: (.*)$')
PID_LEAVE = re.compile(r'^No longer want ([a-zA-Z0-9._:]+) \(pid (\d+)\): .*$')
PID_DEATH = re.compile(r'^Process ([a-zA-Z0-9._:]+) \(pid (\d+)\) has died.?$')

ADB_LOG_REGEX_EXP = 'date,time,process,thread,level,tag,message="(.\S*) *(.\S*) *(\d*) *(\d*) *([A-Z]) *([^:]*): *(.*?)$"'

BUG_LINE = re.compile(r'.*nativeGetEnabledTags.*')
BACKTRACE_LINE = re.compile(r'^#(.*?)pc\s(.*?)$')
RULES = {
    # StrictMode policy violation; ~duration=319 ms: android.os.StrictMode$StrictModeDiskWriteViolation: policy=31 violation=1
    re.compile(r'^(StrictMode policy violation)(; ~duration=)(\d+ ms)')
    : r'%s\1%s\2%s\3%s' % (termcolor(RED), RESET, termcolor(YELLOW), RESET),
}


class Adb:
    all = None
    min_level = None
    package_name = None
    tag = None
    header_size = None
    ignored_tag = None

    log_regex = None
    catchall_package = None
    named_processes = None
    pids = None

    adb = None
    processor = None

    def __init__(self):
        pass

    def setup(self, args):
        self.processor = LogProcessor(args.hide_same_tags)

        self.min_level = LOG_LEVELS_MAP[args.min_level.upper()]
        self.all = args.all
        self.ignored_tag = args.ignored_tag
        self.tag = args.tag

        self.package_name = args.package_or_path
        self.processor.setup_condition(tag_keywords=args.tag_keywords)
        if args.yml is not None:
            conf_file_path = get_conf_path(args.yml)
            if not exists(handle_home_case(conf_file_path)):
                exit('you provide conf file path: ' + conf_file_path + ' is not exist!')

            conf_loader = ConfLoader()
            conf_loader.load(conf_file_path)

            yml_package = conf_loader.get_package()
            if yml_package is not None:
                self.package_name.append(yml_package)

            yml_adb_log_regex = conf_loader.get_adb_log_line_regex()
            if yml_adb_log_regex is not None:
                self.log_regex = LogRegex(yml_adb_log_regex)

            self.processor.setup_condition(tag_keywords=conf_loader.get_tag_keyword_map())
            self.processor.setup_trans(trans_msg_map=conf_loader.get_trans_msg_map(),
                                       trans_tag_map=conf_loader.get_trans_tag_map(),
                                       hide_msg_list=conf_loader.get_hide_msg_list())

            unique_hightlist = set()
            if conf_loader.get_highlight_list():
                unique_hightlist = unique_hightlist.union(set(conf_loader.get_highlight_list()))
            if args.keyword:
                unique_hightlist = list(unique_hightlist.union(set(args.keyword)))
            print(unique_hightlist)
            self.processor.setup_highlight(highlight_list=unique_hightlist)
            self.processor.setup_separator(separator_rex_list=conf_loader.get_separator_regex_list())

        if self.log_regex is None:
            self.log_regex = LogRegex(ADB_LOG_REGEX_EXP)

        base_adb_command = ['adb']
        if args.device_serial:
            base_adb_command.extend(['-s', args.device_serial])
        if args.use_device:
            base_adb_command.append('-d')
        if args.use_emulator:
            base_adb_command.append('-e')

        if args.current_app:
            system_dump_command = base_adb_command + ["shell", "dumpsys", "activity", "activities"]
            system_dump = subprocess.Popen(system_dump_command, stdout=PIPE, stderr=PIPE).communicate()[0]
            running_package_name = re.search(".*TaskRecord.*A[= ]([^ ^}]*)", system_dump).group(1)
            self.package_name.append(running_package_name)

        if len(self.package_name) == 0:
            self.all = True

        # Store the names of packages for which to match all processes.
        self.catchall_package = list(filter(lambda package: package.find(":") == -1, self.package_name))
        # Store the name of processes to match exactly.
        named_processes = filter(lambda package: package.find(":") != -1, self.package_name)
        # Convert default process names from <package>: (cli notation) to <package> (android notation) in the exact names match group.
        self.named_processes = map(lambda package: package if package.find(":") != len(package) - 1 else package[:-1],
                                   named_processes)

        self.header_size = args.tag_width + 1 + 3 + 1  # space, level, space

        # Only enable GC coloring if the user opted-in
        if args.color_gc:
            # GC_CONCURRENT freed 3617K, 29% free 20525K/28648K, paused 4ms+5ms, total 85ms
            key = re.compile(
                r'^(GC_(?:CONCURRENT|FOR_M?ALLOC|EXTERNAL_ALLOC|EXPLICIT) )(freed <?\d+.)(, \d+% free \d+./\d+., )(paused \d+ms(?:\+\d+ms)?)')
            val = r'\1%s\2%s\3%s\4%s' % (termcolor(GREEN), RESET, termcolor(YELLOW), RESET)

            RULES[key] = val

        adb_command = base_adb_command[:]
        adb_command.append('logcat')
        adb_command.extend(['-v', 'brief'])
        adb_command.extend(['-v', 'threadtime'])

        # Clear log before starting logcat
        if args.clear_logcat:
            adb_clear_command = list(adb_command)
            adb_clear_command.append('-c')
            adb_clear = subprocess.Popen(adb_clear_command)

            while adb_clear.poll() is None:
                pass

        if sys.stdin.isatty():
            self.adb = subprocess.Popen(adb_command, stdout=PIPE, stderr=STDOUT, stdin=PIPE)
        else:
            self.adb = FakeStdinProcess()

        self.pids = set()

        ps_command = base_adb_command + ['shell', 'ps']
        ps_pid = subprocess.Popen(ps_command, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        while True:
            try:
                line = ps_pid.stdout.readline().decode('utf-8', 'replace').strip()
            except KeyboardInterrupt:
                break
            if len(line) == 0:
                break

            pid_match = PID_LINE.match(line)
            if pid_match is not None:
                pid = pid_match.group(1)
                proc = pid_match.group(2)
                if proc in self.catchall_package:
                    self.pids.add(pid)

    def loop(self):
        app_pid = None
        tempFileName = time.strftime("%m-%d#%H:%M:%S-tmplog.txt", time.localtime(time.time()))
        print(tempFileName)
        templog = None
        try:
            templog = open(tempFileName, 'w')
        except:
            print("temp log file create fail, log will not be saved")

        while self.adb.poll() is None:
            try:
                line = self.adb.stdout.readline()
                if templog:
                    templog.write(line)
            except KeyboardInterrupt:
                if templog:
                    templog.flush()
                    templog.close()
                break

            if len(line) == 0:
                break

            line = line.decode('utf-8', 'replace').strip()
            if len(line) == 0:
                continue

            bug_line = BUG_LINE.match(line)
            if bug_line is not None:
                continue

            date, detailtime, level, tag, owner, thread, message = self.log_regex.parse(line)
            if message is None:
                # print 'message is none with %s' % line
                continue

            tag = tag.strip()
            start = parse_start_proc(line)
            if start:
                line_package, target, line_pid, line_uid, line_gids = start
                if self.match_packages(line_package):
                    self.pids.add(line_pid)

                    app_pid = line_pid

                    linebuf = '\n'
                    linebuf += colorize(' ' * (self.header_size - 1), bg=WHITE)
                    linebuf += indent_wrap(' Process %s created for %s\n' % (line_package, target))
                    linebuf += colorize(' ' * (self.header_size - 1), bg=WHITE)
                    linebuf += ' PID: %s   UID: %s   GIDs: %s' % (line_pid, line_uid, line_gids)
                    linebuf += '\n'
                    print(linebuf)

            dead_pid, dead_pname = self.parse_death(tag, message)
            if dead_pid:
                self.pids.remove(dead_pid)
                linebuf = '\n'
                linebuf += colorize(' ' * (self.header_size - 1), bg=RED)
                linebuf += ' Process %s (PID: %s) ended' % (dead_pname, dead_pid)
                linebuf += '\n'
                print(linebuf)

            # Make sure the backtrace is printed after a native crash
            if tag == 'DEBUG':
                bt_line = BACKTRACE_LINE.match(message.lstrip())
                if bt_line is not None:
                    message = message.lstrip()
                    owner = app_pid

            # print '%s %s %s' % (owner, self.pids, tag)
            if not self.all and owner not in self.pids:
                continue
            if level in LOG_LEVELS_MAP and LOG_LEVELS_MAP[level] < self.min_level:
                continue
            if self.ignored_tag and tag_in_tags_regex(tag, self.ignored_tag):
                continue
            if self.tag and not tag_in_tags_regex(tag, self.tag):
                continue

            msg_key, linebuf, match_precondition = self.processor.process_decode_content(line, date, detailtime, level, tag, owner,
                                                                                         thread,
                                                                                         message)
            if not match_precondition or linebuf is None:
                continue

            if msg_key is not None:
                print('')
                print_unicode(u''.join(colorize(msg_key + ": ", fg=allocate_color(msg_key))).encode('utf-8').lstrip())

            print_unicode(u''.join(linebuf).encode('utf-8').lstrip())

    def match_packages(self, token):
        if len(self.package_name) == 0:
            return True
        if token in self.named_processes:
            return True
        index = token.find(':')
        return (token in self.catchall_package) if index == -1 else (token[:index] in self.catchall_package)

    def parse_death(self, _tag, _message):
        if _tag != 'ActivityManager':
            return None, None
        kill = PID_KILL.match(_message)
        if kill:
            _pid = kill.group(1)
            package_line = kill.group(2)
            if self.match_packages(package_line) and _pid in self.pids:
                return _pid, package_line
        leave = PID_LEAVE.match(_message)
        if leave:
            _pid = leave.group(2)
            package_line = leave.group(1)
            if self.match_packages(package_line) and _pid in self.pids:
                return _pid, package_line
        death = PID_DEATH.match(_message)
        if death:
            _pid = death.group(2)
            package_line = death.group(1)
            if self.match_packages(package_line) and _pid in self.pids:
                return _pid, package_line
        return None, None  # This is a ducktype of the subprocess.Popen object


class FakeStdinProcess:
    def __init__(self):
        self.stdout = sys.stdin

    @staticmethod
    def poll():
        return None


def parse_start_proc(_line):
    _start = PID_START_5_1.match(_line)
    if _start is not None:
        _line_pid, _line_package, _target = _start.groups()
        return _line_package, _target, _line_pid, '', ''
    _start = PID_START.match(_line)
    if _start is not None:
        _line_package, _target, _line_pid, _line_uid, _line_gids = _start.groups()
        return _line_package, _target, _line_pid, _line_uid, _line_gids
    _start = PID_START_DALVIK.match(_line)
    if _start is not None:
        _line_pid, _line_package, _line_uid = _start.groups()
        return _line_package, '', _line_pid, _line_uid, ''
    return None


def tag_in_tags_regex(_tag, tags):
    return any(re.match(r'^' + t + r'$', _tag) for t in map(str.strip, tags))



#!/usr/bin/env python
# coding: utf-8

"""
Copyright (C) 2017 Jacksgong(jacksgong.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

__author__ = 'jacks.gong'


class BashColors:
    def __init__(self):
        pass

    HEADER = '\033[95m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    END = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


def print_header(msg):
    print(BashColors.HEADER + msg + BashColors.END)


def print_exit(msg):
    print(BashColors.FAIL + msg + BashColors.END)


def print_error(msg):
    print(BashColors.FAIL + msg + BashColors.END)


def print_tips(msg):
    print(BashColors.UNDERLINE + msg + BashColors.END)


def print_warn(msg):
    print(BashColors.WARNING + msg + BashColors.END)


def print_key(msg):
    print(BashColors.GREEN + msg + BashColors.END)


def print_blue(msg):
    print(BashColors.BLUE + msg + BashColors.END)


def print_content_tips(msg):
    msg = BashColors.UNDERLINE + msg + BashColors.END
    print(msg)
    return msg + "\n"


def print_content_header(msg):
    msg = BashColors.HEADER + msg + BashColors.END
    print_content(msg)
    return msg + "\n"


def print_content(msg):
    print(msg)
    return msg


# -------------- color -----------------------------
RESET = '\033[0m'
BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)


def termcolor(fg=None, bg=None):
    codes = []
    if fg is not None: codes.append('3%d' % fg)
    if bg is not None: codes.append('10%d' % bg)
    return '\033[%sm' % ';'.join(codes) if codes else ''


def colorize(message, fg=None, bg=None):
    if fg is None:
        if bg == BLACK:
            fg = WHITE
        else:
            fg = BLACK

    return termcolor(fg, bg) + message + RESET


TAGTYPES = {
    'V': colorize(' V ', fg=WHITE, bg=BLACK),
    'D': colorize(' D ', fg=BLACK, bg=BLUE),
    'I': colorize(' I ', fg=BLACK, bg=GREEN),
    'W': colorize(' W ', fg=BLACK, bg=YELLOW),
    'E': colorize(' E ', fg=BLACK, bg=RED),
    'F': colorize(' F ', fg=BLACK, bg=RED),
}

# for random color
KNOWN_TAGS = {
    'dalvikvm': WHITE,
    'Process': WHITE,
    'ActivityManager': BLUE,
    'ActivityThread': WHITE,
    'AndroidRuntime': CYAN,
    'jdwp': WHITE,
    'StrictMode': WHITE,
    'DEBUG': YELLOW,
}
LAST_USED = [RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN]
LOG_FLOW_KEY_LAST_USED = [RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN]


def allocate_color(_key, loop_color=LAST_USED):
    if _key not in KNOWN_TAGS:
        KNOWN_TAGS[_key] = loop_color[0]

    _color = KNOWN_TAGS[_key]
    if _color in LAST_USED:
        loop_color.remove(_color)
        loop_color.append(_color)
    return _color



#!/usr/bin/python -u
# coding: utf-8

"""
Copyright (C) 2017 Jacksgong(jacksgong.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import re
from os.path import exists

from ducat.confloader import ConfLoader
from ducat.helper import get_conf_path, print_unicode
from ducat.logprocessor import LogProcessor
from ducat.terminalcolor import colorize, allocate_color, termcolor, BLACK, RED, RESET

TIME_REGEX = r'\d{2}-\d{2} \d{2}:\d{2}:\d{2}\.\d+'

class LogFileParser:
    filePaths = []
    valid = False
    processor = None
    hideSameTags = None
    keywords = None

    def __init__(self, file_paths, hide_same_tags, keywords):
        self.filePaths = file_paths
        self.hideSameTags = hide_same_tags
        self.keywords = keywords

    def setup(self, yml_file_name):
        for path in self.filePaths:
            if not exists(path):
                exit("log path: %s is not exist!" % path)
        self.processor = LogProcessor(self.hideSameTags)

        loader = ConfLoader()
        loader.load(get_conf_path(yml_file_name))

        self.processor.setup_trans(trans_msg_map=loader.get_trans_msg_map(),
                                   trans_tag_map=loader.get_trans_tag_map(),
                                   hide_msg_list=loader.get_hide_msg_list())
        self.processor.setup_separator(separator_rex_list=loader.get_separator_regex_list())
        highlight_list = loader.get_highlight_list()
        unique_hightlist = set()
        if highlight_list:
            unique_hightlist = unique_hightlist.union(set(highlight_list))
        if self.keywords:
            unique_hightlist = list(unique_hightlist.union(set(self.keywords)))
        print(unique_hightlist)
        self.processor.setup_highlight(list(unique_hightlist))
        self.processor.setup_condition(tag_keywords=loader.get_tag_keyword_map())
        self.processor.setup_regex_parser(regex_exp=loader.get_log_line_regex())

    def color_line(self, line, fileName):
        msg_key, line_buf, match_precondition = self.processor.process(line)
        if not match_precondition:
            return False

        if msg_key is not None:
            print('')
            print_unicode(u''.join(colorize(msg_key + ": ", fg=allocate_color(msg_key))).encode('utf-8').lstrip())

        if fileName:
            print_unicode(u''.join(fileName + ":" + line_buf).encode('utf-8').lstrip())
        else:
            print_unicode(u''.join(line_buf).encode('utf-8').lstrip())
        return True


    def process(self):
        for logPath in self.filePaths:
            stream = open(logPath, "r")
            indexOfLastBackSlant = logPath.rfind("/")
            if not indexOfLastBackSlant:
                indexOfLastBackSlant = 0
            else:
                indexOfLastBackSlant = indexOfLastBackSlant + 1
            fileName = logPath[indexOfLastBackSlant:]
            if len(self.filePaths) == 1:
                fileName = None
            newLine = stream.readline()
            hasMatchLine = False
            while newLine:
                newLineResult = self.color_line(newLine, fileName)
                hasMatchLine = newLineResult or hasMatchLine
                newLine = stream.readline()
            if hasMatchLine and len(self.filePaths) > 1:
                print_unicode(termcolor(fg=BLACK, bg=RED) + "===================文件" + logPath + "结束分割线======================" + RESET)
                                              


#!/usr/bin/python -u
# coding:utf-8

"""
Copyright (C) 2017 Jacksgong(jacksgong.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from distutils import errors
import os
import re
import sys

from os import environ, getcwd

from os.path import exists

__author__ = 'JacksGong'

LOG_LEVELS = 'VDIWEF'
LOG_LEVELS_MAP = dict([(LOG_LEVELS[i], i) for i in range(len(LOG_LEVELS))])

NO_HOME_PATH = re.compile(r'~/(.*)')
HOME_PATH = environ['HOME']


# get the home case path
def handle_home_case(path):
    path = path.strip()
    if path.startswith('~/'):
        path = HOME_PATH + '/' + NO_HOME_PATH.match(path).groups()[0]
    return path


# 调整is path的逻辑，支持文件名模糊匹配, 对于正则表达式，返回匹配的文件列表
def getFilePaths(logs):
    fileList = list()
    for logPath in logs:
        indexOfLastBackSlant = logPath.rfind("/")
        candidateFiles = list() 
        fileName = logPath
        logFileDir = os.getcwd()
        if (indexOfLastBackSlant == -1):
            # 单文件，直接返回当前文件
            candidateFiles = os.listdir(logFileDir)
        else:
            # 包含目录
            logFileDir = logPath[0 : indexOfLastBackSlant]
            candidateFiles = os.listdir(logFileDir)
            fileName = logPath[indexOfLastBackSlant + 1 :]
        for subFile in candidateFiles:
            if re.match(fileName, subFile):
                fileList.append(os.path.join(logFileDir, subFile))
    return fileList


def get_conf_path(conf_name):
    if not conf_name.endswith('.yml'):
        conf_name = conf_name + '.yml'

    cur_path_yml = '%s/%s' % (getcwd(), conf_name)
    if exists(cur_path_yml):
        result = cur_path_yml
    else:
        result = '~/.okcat/' + conf_name

    print('using config on %s' % result)
    return result


def print_unicode(line):
    if sys.version_info >= (3, 0):
        print(bytes.decode(line))
    else:
        print(line)

def line_rstrip(line):
    if sys.version_info >= (3, 0):
        return line.rstrip()
    else:
        return line.decode('utf-8', errors='ignore').rstrip()
def init_baseyml():
    config_path = HOME_PATH
    config_path = os.path.join(config_path, ".okcat")
    if not os.path.exists(config_path):
        os.makedirs(config_path)
    if os.path.isdir(config_path):
        # init base.yml
        print("init base yml")
        
        with open(os.path.join(config_path, "base.yml"), 'w') as baseyml:
            baseyml.write("# PLEASE DO NOT EDIT THIS FILE, IT WILL BE OVERWRITE BY DUCAT!! 请不要编辑此文件，此文件随时会被覆盖")
            baseyml.write("\n")
            baseyml.write("tag-keyword-list:")
            baseyml.write("\n")
            baseyml.write("  'AndroidRuntime': W,E")
            baseyml.write("\n")
            baseyml.write("  'ActivityManager': E")
            baseyml.write("\n")
            baseyml.write("  'DEBUG': F")
            baseyml.write("\n")
            baseyml.write('log-line-regex: \'date,time,process,thread,level,tag,message = "(.\S*) *(.\S*) *(\d*) *(\d*) *([A-Z]) *([^:]*):\s*(.*)$"\'')
            baseyml.write("\n")
            baseyml.write('trans-msg-map:')
            baseyml.write("\n")
            baseyml.write('  \'event=SEND_EVENT.*name":"(\w*)\': "上报事件"')
            baseyml.write("\n")
            baseyml.write('  \'^((?!FinishHandleDirectives)\w*)\s*receive directive:\': "收到指令"')
            baseyml.write("\n")
            baseyml.write('  \'Start proc (\d+):([\w\.]+)\': "启动进程"')
            baseyml.write("\n")
            baseyml.write('  \'eventType:ACTIVITY_RESUMED, packageName:([\w\.]+)\': "页面进前台"')
            baseyml.write("\n")
            baseyml.write('  \'eventType:ACTIVITY_PAUSED, packageName:([\w\.]+)\': "页面进后台"')
            baseyml.write("\n")
            baseyml.write('  \'FATAL EXCEPTION:\': "Java崩溃"')
            baseyml.write("\n")
            baseyml.write('  \'START u\d+ \{flg=0x[01]+ cmp=([\w\.]+)\': "启动页面"')
            baseyml.write("\n")
            baseyml.write('  \'Fatal signal \d+\s+.*pid\s+(\d+)\s+(.*)\': "Native崩溃"')
            baseyml.write("\n")
            baseyml.write('  \'ANR in ([\w\.]+)\': "无响应ANR"')
            baseyml.write("\n")
            baseyml.write('  \'onPartialResult = (\S+)\': "用户query"')
            baseyml.write("\n")
            baseyml.flush()
            baseyml.close()
        
    else:
        print("init base yml fail, ~/.okcat is not dir")
        


#!/usr/bin/python -u
# coding:utf-8

"""
Copyright (C) 2017 Jacksgong(jacksgong.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import argparse
from sys import argv

from ducat.adb import Adb
from ducat.helper import LOG_LEVELS, getFilePaths, init_baseyml
from ducat.logfile_parser import LogFileParser
from ducat.terminalcolor import print_tips, print_blue, print_warn, print_header, print_exit

__author__ = 'JacksGong'
__version__ = '1.1.6'
__description__ = 'This python script used for combine several Android projects to one project.'

def main():
    print("-------------------------------------------------------")
    print("                  OkCat v" + __version__)
    print("")
    print("Thanks for using okcat! Now, the doc is available on: ")
    print_blue("        https://github.com/Jacksgong/okcat")
    print("")
    print("                   Have Fun!")
    print("-------------------------------------------------------")

    parser = argparse.ArgumentParser(description='Filter logcat by package name')
    parser.add_argument('package_or_path', nargs='*',
                        help='This can be Application package name(s) or log file path(if the file from path is exist)')
    parser.add_argument('-y', '--yml_file_name', dest='yml', help='Using yml file you config on ~/.okcat folder')
    parser.add_argument('--hide-same-tags', dest='hide_same_tags', action='store_true',
                        help='Do not display the same tag name')

    # following args are just for parser
    parser.add_argument('-k', '--keyword', dest='keyword', action='append', help='You can filter you care about log by this keyword(s)')

    # following args are just for adb
    parser.add_argument('-w', '--tag-width', metavar='N', dest='tag_width', type=int, default=23,
                        help='Width of log tag')
    parser.add_argument('-l', '--min-level', dest='min_level', type=str, choices=LOG_LEVELS + LOG_LEVELS.lower(),
                        default='V', help='Minimum level to be displayed')
    parser.add_argument('--color-gc', dest='color_gc', action='store_true', help='Color garbage collection')
    parser.add_argument('--current', dest='current_app', action='store_true',
                        help='Filter logcat by current running app')
    parser.add_argument('-s', '--serial', dest='device_serial', help='Device serial number (adb -s option)')
    parser.add_argument('-d', '--device', dest='use_device', action='store_true',
                        help='Use first device for log input (adb -d option)')
    parser.add_argument('-e', '--emulator', dest='use_emulator', action='store_true',
                        help='Use first emulator for log input (adb -e option)')
    parser.add_argument('-c', '--clear', dest='clear_logcat', action='store_true',
                        help='Clear the entire log before running')
    parser.add_argument('-t', '--tag', dest='tag', action='append', help='Filter output by specified tag(s)')
    parser.add_argument('-tk', '--tag_keywords', dest='tag_keywords', action='append',
                        help='Filter output by specified tag keyword(s)')
    parser.add_argument('-i', '--ignore-tag', dest='ignored_tag', action='append',
                        help='Filter output by ignoring specified tag(s)')
    parser.add_argument('-a', '--all', dest='all', action='store_true', default=False,
                        help='Print all log messages')

    # help
    if len(argv) == 2 and argv[1] == 'help':
        exit()

    args = parser.parse_args()

    candidate_path = args.package_or_path
    file_paths = getFilePaths(candidate_path)



    init_baseyml()
    if args.yml is None:
        print("parse file is None, use base.yml")
        print("you can use '-y=conf-name' to provide config file to parse this log file.")
        print("The config file is very very simple! More detail about config file please move to : https://github.com/Jacksgong/okcat")
        print("")
        print("-------------------------------------------------------")
        args.yml = "base.yml"
    if file_paths :
        print("parse file")

        parser = LogFileParser(file_paths, args.hide_same_tags, args.keyword)
        parser.setup(args.yml)
        parser.process()
    else:
        is_interrupt_by_user = False
        print("parse adb")

        _adb = Adb()
        _adb.setup(args)
        try:
            _adb.loop()
        except KeyboardInterrupt:
            is_interrupt_by_user = True

        if not is_interrupt_by_user:
            print_warn('ADB CONNECTION IS LOST.')

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

"""
Copyright (C) 2017 Jacksgong(jacksgong.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import yaml

from ducat.helper import handle_home_case, get_conf_path

__author__ = 'JacksGong'


class ConfLoader:
    yml_conf = None
    from_yml_conf = None

    def __init__(self):
        pass

    def load(self, yml_file_path):
        with open(handle_home_case(yml_file_path), 'r') as stream:
            try:
                self.yml_conf = yaml.full_load(stream)
                from_conf_path = self.get_from()
                if from_conf_path is not None:
                    self.from_yml_conf = ConfLoader()
                    self.from_yml_conf.load(get_conf_path(from_conf_path))

                    # print(u'find yml configuration on %s:' % yml_file_path)
                    # self.dump()

            except yaml.YAMLError as exc:
                print(exc)

    def get_from(self):
        return self.get_value('from')

    def get_package(self):
        return self.get_value('package')

    def get_tag_keyword_map(self):
        return self.get_value('tag-keyword-list')

    def get_trans_msg_map(self):
        return self.get_value('trans-msg-map')

    def get_trans_tag_map(self):
        return self.get_value('trans-tag-map')

    def get_hide_msg_list(self):
        return self.get_value('hide-msg-list')

    def get_highlight_list(self):
        return self.get_value('highlight-list')

    def get_log_line_regex(self):
        return self.get_value('log-line-regex')

    def get_adb_log_line_regex(self):
        return self.get_value('log-line-regex')

    def get_separator_regex_list(self):
        return self.get_value('separator-regex-list')

    def get_value(self, keyword):
        if keyword not in self.yml_conf:
            if keyword != 'from' and self.from_yml_conf is not None:
                return self.from_yml_conf.get_value(keyword)
            else:
                return None
        return self.yml_conf[keyword]

    def dump(self):
        print('from: %s' % self.get_from())
        print('package: %s' % self.get_package())
        print('log-line-regex: %s' % self.get_log_line_regex())
        print('adb-log-line-regex: %s' % self.get_adb_log_line_regex())
        self.dump_unicode_map('tag-keyword-list')
        self.dump_unicode_map('trans-msg-map')
        self.dump_unicode_map('trans-tag-map')
        self.dump_list('hide-msg-list')
        self.dump_list('highlight-list')
        self.dump_list('separator-regex-list')

    def dump_unicode_map(self, map_key):
        unicode_map = self.get_value(map_key)
        if unicode_map is None:
            print('%s: None' % map_key)
        else:
            print('%s:' % map_key)
            for key in unicode_map:
                print(u'    "%s" : "%s"' % (key, unicode_map[key]))

    def dump_list(self, list_key):
        cur_list = self.get_value(list_key)
        if cur_list is None:
            print('%s: None' % list_key)
        else:
            print('%s: ' % list_key)
            for value in cur_list:
                print('    - %s' % value)



#!/usr/bin/python -u

"""
Copyright (C) 2017 Jacksgong(jacksgong.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from setuptools import setup, find_packages

# Get the long description from the README file
# noinspection PyArgumentList
setup(
    name="DuCat",
    version="1.0.1",
    packages=find_packages(exclude=['demo-conf', 'arts']),

    # Project uses reStructuredText, so ensure that the docutils get
    # installed or upgraded on the target machine
    install_requires=['PyYAML>=3.12'],

    # metadata for upload to PyPI
    author="Zhentao",
    author_email="chenzhentao70@gmail.com",
    description="An powerful log processor BASE ON OkCat",
    long_description='More detail please move to https://github.com/TokenChen/okcat',
    license="Apache2",
    keywords="okcat log 'log processor' 'log filter'",
    url="https://github.com/TokenChen/okcat",

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: Apache Software License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    entry_points={
        'console_scripts': [
            'ducat=ducat:main'
        ]
    }
)
